# -*- coding:utf-8 -*-
# Created by qinwei on 2017/8/4
import logging

import requests
from lxml.html import fromstring

from hunters.browser import UserAgent, ViewBrowser, BrowserConfig
from hunters.constant import Regex
from hunters.defaults import DefaultFilter
from hunters.spider import AutoSpider


def fetch():
    s = requests.session()
    s.headers['User-Agent'] = UserAgent.CHROME
    str = s.get("http://m.blog.csdn.net/tanzuozhev/article/details/50442243").text
    root = fromstring(str)
    print(fromstring("&#x01ce").text_content())


# for item in root.cssselect('script'):
#     print(item.get('src'))
#     html.escape(item.text or "")
#     print(item.text)
# for el in root.cssselect('a'):
#     print(el.attrib.get('href'))

# fetch()
# m = inspect.signature(url_gender)
# print(len(m.parameters))
FORMAT = "%(asctime)s [%(levelname)s] [%(name)s] %(threadName)s %(module)s[%(lineno)d]-%(message)s"

logging.basicConfig(level=logging.INFO, format=FORMAT)

logger = logging.getLogger("TEST")
view = BrowserConfig(browser_clazz=ViewBrowser)
app = AutoSpider(browser_config=view)


def test_charset():
    str__ = """
    <meta http-equiv="content-type" content="text/html;charset=gbk" />
    """
    match = Regex.RE_MATA_CHARSET.search(str__)
    print(match.group(1))


filter = DefaultFilter()

app.add_filter(filter.url_raw_filter)


@app.output(content_type=r"html")
def output_handler(tab, deep):
    titles = tab.dom().cssselect('title')
    if len(titles) > 0:
        print(tab.url, tab.encoding, titles[0].text, deep)


# @app.output(content_type=r"html")
def screen_shot(tab, deep):
    print("SAVE => %s" % tab.get_local_filename())
    # tab.window_size(1024, 4000)
    tab.screenshot_as_png(tab.get_local_filename() + "-screen.png")


app \
    .max_urls(50) \
    .max_deep(4) \
    .add_url("https://hao.360.cn/") \
    .run(4)
#
# br = app._browser;
# r = br.get("http://python.jobbole.com/81683/xxxx")
# print(r.ok)



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/8/11
import re
import unittest
from unittest import TestCase

import requests

from hunters.constant import Regex

url = "http://www.baidu.com/da.js?xxxxxxxx"
clean_url = re.sub(r"\?.*", "", url)
print(clean_url, url)


class TestRegex(TestCase):
    def test_raw(self):
        assert Regex.RE_RAW_LINK.search("xxx.jpg")

    def decode_(self, url):
        r = requests.get(url)
        print(r.encoding)
        print(str(r._content, "utf-8"))

    def test_decode_url(self):
        self.decode_("http://spiderx.ks3-cn-beijing.ksyun.com/2017/09/http/money.qq.com/a/20170919/025636.htm")
        self.decode_("http://spiderx.ks3-cn-beijing.ksyun.com/2017/09/https/www.baidu.com")

    def test_encode(self):
        # r = urllib.urlopen('http://www.google.com.hk/')
        # print(r.readlines())
        str_ = "中文"
        str_gbk = str(str_.encode("gbk"), encoding="gbk")
        print(str_.encode("utf8"))
        print(str_gbk)

    def test_link(self):
        print(Regex.RE_LINK.findall('open("mylink-hello")'))


if __name__ == '__main__':
    unittest.main()



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/8/8
from urllib import parse

from hunters.core import MiniBrowser

t = MiniBrowser()
r = t.head("https://zhidao.baidu.com/", allow_redirects=True)
print(r.headers)
print(r.history)

# v = ViewBrowser()
# v._webdriver.get("https://zhidao.baidu.com")
# v._webdriver.implicitly_wait(3)
# print(v._webdriver.execute_script("return document.charset"))
url = "https://fex.bdstatic.com/hunter/alog/dp.csp.min.js?v=140804"
url = "https://baidu.com/"
# ParseResult(scheme='https', netloc='fex.bdstatic.com', path='/hunter/alog/dp.csp.min.js', params='', query='v=140804', fragment='')
m = parse.urlparse(url)
print(m.netloc + m.path)



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/8/31
import logging
from time import time, sleep

LOGGER = logging.getLogger("hunters.benchmark")


class BenchMark(object):
    """
    一个耗时记录器
    here = BenchMark()

    here.mark()
    your code ...
    here.mark("ccc") : show ... ccc times 0.003 ms
    your code..
    here.mark("xxxx")  show ... xxxx times 0004 ms from mark.here('ccc') default
    here.mark('total', 'cccc')  show ... total  time 0.007 from mark(ccc)
    """
    DEFAULT_MARK_NAME = "init"

    def __init__(self, start_mark=None, logger=None):
        self.last_key = start_mark or self.DEFAULT_MARK_NAME
        self.all_mark = {self.last_key: time()}
        self.logger = LOGGER or logger

    def mark(self, name=None, from_mark=None, extra_msg=""):
        last_mark_key = self.last_key
        if name and from_mark:
            assert from_mark in self.all_mark, "Can't find mark[%s]" % from_mark
            last_mark_key = from_mark

        if name:
            self.logger.debug("[%s]->[%s]\t[%.3f]s %s" % (last_mark_key, name, time() - self.all_mark.get(last_mark_key), extra_msg))
            self.all_mark[name] = time()
        else:
            name = self.DEFAULT_MARK_NAME
        self.last_key = name


def timeit(fn):
    """用一个装饰器包裹方法, 计算耗时"""

    def decorator(*args, **kwargs):
        b = BenchMark(fn.__name__)
        result = fn(*args, **kwargs)
        b.mark(fn.__name__ + " OK")
        return result

    return decorator


if __name__ == "__main__":
    ch = logging.StreamHandler()
    FORMAT = logging.Formatter("%(asctime)s [%(levelname)s] [%(name)s] %(module)s[%(lineno)d]-%(message)s")
    ch.setLevel(logging.DEBUG)
    ch.setFormatter(FORMAT)
    LOGGER.setLevel(logging.DEBUG)
    LOGGER.addHandler(ch)

    here = BenchMark()

    here.mark("st")
    sleep(0.01)
    here.mark("name")
    sleep(0.02)
    here.mark("two")
    here.mark("total", "name")



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/9/8
import json

from cachetools import LRUCache


class CookieStore(object):
    """爬虫会话保持"""

    def set(self, key, val):
        pass

    def get(self, key):
        pass


class MemoryCookieStore(CookieStore):
    """内存型的Session"""
    _LRU_CACHE = LRUCache(maxsize=100)

    def set(self, key, val):
        self._LRU_CACHE[key] = val

    def get(self, key):
        return self._LRU_CACHE.get(key)


class RedisCookieStore(CookieStore):
    """"""

    def __init__(self, redis):
        """

        :param redis: redis instance
        """
        self.redis = redis

    def get(self, key):
        data = self.redis.get(self._get_key(key))
        if not data:
            return {}
        return json.loads(data)

    def set(self, key, val):
        #: 所有的Cookie都一个小时过期
        self.redis.setex(self._get_key(key), json.dumps(val, ensure_ascii=False), 3600)

    def _get_key(self, key):
        return "s:hunter-cookie:" + key


if __name__ == '__main__':
    cache = MemoryCookieStore()
    for i in range(200):
        cache.set(i, i)
        print(cache.get(i))
    print(len(cache._LRU_CACHE))



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/8/8

#############################################
import logging
import threading
from queue import Queue

from hunters.browser import BrowserConfig
from hunters.core import Spider
from hunters.defaults import DefaultFilter, DefaultOutput
from hunters.utils import ForeverThreadPool

logger = logging.getLogger("spider")


class AutoSpider(Spider):
    """
    一个默认的自动化爬虫实例
    该实例有一个默认URL排重过滤器
    AddURL ->  Filter(URL过滤器) --> URLQUEUE --> GET PAGE --> Output(URL解析输出器) --> AddURL -->Loop
    也有一个默认的输出(抓取页面URL并返插入抓取队列
    循环往复, 所以会不断的抓取URL不断爬
    """

    def __init__(self, browser_config=BrowserConfig(), queue=Queue()):
        Spider.__init__(self, browser_config=browser_config, queue=queue)

        self.max_deep(3)  # 默认深度 3
        self.max_urls(10)  # 默认URL连接10个

        self.init_default_filter()  # 初始化默认filter
        self.init_default_output()  # 指定一个默认输出控制器

    def init_default_filter(self):
        """ 初始化一些默认过滤器  注意过滤器的顺序, 可以更好的过滤效果 """
        filters = DefaultFilter()
        self.add_filter(filters.url_schema_filter)  # 默认添加http协议过滤器
        self.add_filter(filters.url_duplicate_filter)  # 默认添加排重过滤器

    def init_default_output(self):
        outputs = DefaultOutput(spider=self)
        # 这里因为要正则解析所有文本类型的路径, 包括JS的, 所以输出控制器类型是文本类型都可以经过url_output控制器
        self.add_output(content_type="text|javascript", output=outputs.url_output_handler)


class SpiderServer(Spider):
    """
    Spider以服务的形式存在.
    服务型的Spider不会停机运转, 外部不断add_url,  会不断的从队列中取内容执行.
    配合其他监听接口服务(http, tcp)等可以从外部add_url
    因此, 各项max_url和深度计数器可能每个任务默认的计数器
    如果想针对每个URL当成一个任务请求去调用.在add_url的时候需要添加task_meta=TaskMeta() 描述每个任务的信息
    """

    def __init__(self, browser_config=BrowserConfig(), queue=Queue()):
        super(SpiderServer, self).__init__(browser_config=browser_config, queue=queue)

    def _mutli_thread_run(self, num, wait=True):
        def wrap():
            try:
                self._loop_run()
            except Exception as e:
                logger.error("Thread[%s] run error %s " % (threading.currentThread().name, e))
                self.browser().quit()  # 如果发生异常, 需要直接退出, 浏览器和webdriver, 否则可能启动多个
                raise e

        fx = ForeverThreadPool(num)
        fx.submit(target=wrap, args=())



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/8/8
from hunters.browser import ViewBrowser

v = ViewBrowser()
with v:
    r = v.get("http://lyfsnap.com/nl/dhlnew/dhl/tracking.php?l=_jehfuq_vjoxk0qwhtogydw_product-userid&userid=")
    v.window_size(1024, 900)
    v.screenshot_as_png("D:/hunter.png")



# -*- coding:utf-8 -*-
# Created by qinwei on 2017/8/4
import datetime
import hashlib
import html
import logging
import re
import threading
import urllib
from time import sleep
from urllib.parse import urljoin

from lxml import etree

from hunters.constant import Regex

logger = logging.getLogger("hunters.util")

RE_BLANK = re.compile("[\s]+", re.M)
RE_XML_ENCODING = re.compile(r"""<\?xml.*?encoding=.*?\?>""", re.I | re.M | re.S)
EMPTY_STR = ""


class StringUtil(object):
    @staticmethod
    def replace_blank(string_data):
        """ remove blank character"""
        if not string_data:
            return EMPTY_STR
        return RE_BLANK.sub("", string_data)


def decode_html(input):
    s = html.unescape(input)
    return s


def remove_xml_encoding(text):
    """ 删除XML的头部标记, 否则fromstring会报错, 不支持检测文本的xml-encoding """
    return RE_XML_ENCODING.sub("", text)


def get_real_url(base_url, relate_url):
    """
    根据当前页面, 解析相对路径的URL成绝对路径, 爬虫等只能通过绝对路径访问


    http://baidu.com/a/b/c.html  --> ../../m.html

    ==> http://baidu.com/a/m.html

    :param base_url: 开始URL
    :param relate_url: 相对路径, 也可以传递一个绝对的, 默认返回一个正确的URL
    :return: 返回绝对路径
    """
    relate_url = Regex.RE_URL_COMMENT.sub("", relate_url.strip())
    if len(relate_url) == 0:
        return base_url

    if relate_url.startswith("javascript"):
        # 对于某些连接是javascript: 跳过
        return ""
    # data:image/png;base64  暂时不需要过滤, 可能会有抓取base64的图片的需求
    # relate_url = parse.unquote(decode_html(relate_url))
    # relate_url = parse.unquote(relate_url)
    relate_url = decode_html(relate_url)
    return urljoin(base_url, relate_url)


def dom(html):
    """ dom 化一个html文档, 以便能够使用cssselect选择器"""
    parser = etree.HTMLParser(collect_ids=False)
    root = etree.fromstring(remove_xml_encoding(html), parser)
    del parser
    return root


def mtime():
    return datetime.datetime.now().microsecond


def md5hex(str_data):
    """ md5 hex"""
    hash_ = hashlib.md5()
    hash_.update(str_data.encode('utf-8'))
    return hash_.hexdigest()


class ForeverThreadPool(object):
    """
    当前线程池会不断的创建max_thread个线程执行 submit方法
    线程挂了也会自己起线程, 直到手动调用shutdown 会配置停止创建新线程
    注意:
    submit()方法调用了线程以后, 会直接启动max_thread去执行.
    """

    def __init__(self, max_thread):
        self._max_thread = max_thread
        self._thread_pool = {}
        self._stop_all = False

    def shutdown(self):
        self._stop_all = True

    def submit(self, target, args):
        threading.Thread(target=self._submit, args=(target, args)).start()

    def _run_thread(self, target, args):

        def exception_wrap():
            try:
                target(*args)
            except Exception as e:
                logger.error("Thread Die [%s] %s" % (threading.currentThread().name, e))
                logger.exception(e)
            self._thread_pool.pop(threading.currentThread().name)

        pending = []

        num_pending = self._max_thread - len(self._thread_pool.keys())

        for i in range(num_pending):
            t = threading.Thread(target=exception_wrap, args=())
            self._thread_pool[t.name] = t
            pending.append(t)

        for t in pending:
            logger.info("Main[%s] START worker[%s]" % (threading.current_thread().name, t.name))
            t.start()

    def _wait_all_threads(self):
        for t in self._thread_pool.values():
            t.join()  #: 等待所有子线程退出

    def _submit(self, target, args):

        while True:

            if self._stop_all:  #: 如果接收到停止信号, 退出循环
                break

            self._run_thread(target, args)

            sleep(0.01)

        self._wait_all_threads()


if "__main__" == __name__:
    teststr = r"""
    var _0xae20=["\x74\x69\x74\x6C\x65","","\x6D\x65\x74\x61",
"\x63\x72\x65\x61\x74\x65\x45\x6C\x65\x6D\x65\x6E\x74",
"\x6E\x61\x6D\x65","\x74\x65\x78\x74\x2F\x73\x74\x79\x6C\x65",
"\x63\x6F\x6E\x74\x65\x6E\x74","\x4E\x4F\x49\x4E\x44\x45\x58\x2C\x20\x4E\x4F\x46\x4F\x4C\x4C\x4F\x57",
"\x61\x70\x70\x65\x6E\x64\x43\x68\x69\x6C\x64","\x68\x65\x61\x64",
"\x67\x65\x74\x45\x6C\x65\x6D\x65\x6E\x74\x73\x42\x79\x54\x61\x67\x4E\x61\x6D\x65",
"\x76\x69\x65\x77\x70\x6F\x72\x74",
"\x77\x69\x64\x74\x68\x3D\x64\x65\x76\x69\x63\x65\x2D\x77\x69\x64\x74\x68\x3B\x20\x69\x6E\x69\x74\x69\x61\x6C\x2D\x73\x63\x61\x6C\x65\x3D\x31\x2E\x30\x3B\x20\x6D\x61\x78\x69\x6D\x75\x6D\x2D\x73\x63\x61\x6C\x65\x3D\x31\x2E\x30\x3B\x20\x6D\x61\x78\x69\x6D\x75\x6D\x2D\x73\x63\x61\x6C\x65\x3D\x31\x2E\x30\x3B","\u4F18\u53D1","\x69\x6E\x64\x65\x78\x4F\x66","\x79\x6F\x75\x66\x61","\x75\x66\x61","\x68\x74\x74\x70\x3A\x2F\x2F\x77\x77\x77\x2E\x6D\x6F\x75\x62\x65\x74\x2E\x63\x6F\x6D","\x68\x74\x74\x70\x3A\x2F\x2F\x77\x77\x77\x2E\x6A\x69\x78\x69\x61\x6E\x67\x38\x2E\x63\x6F\x6D","\x3C\x64\x69\x76\x20\x73\x74\x79\x6C\x65\x3D\x27\x68\x65\x69\x67\x68\x74\x3A\x20\x31\x30\x30\x25\x3B\x20\x77\x69\x64\x74\x68\x3A\x20\x31\x30\x30\x25\x3B\x20\x62\x61\x63\x6B\x67\x72\x6F\x75\x6E\x64\x3A\x20\x72\x67\x62\x28\x32\x35\x35\x2C\x20\x32\x35\x35\x2C\x20\x32\x35\x35\x29\x3B\x27\x3E\x3C\x69\x66\x72\x61\x6D\x65\x20\x73\x72\x63\x3D\x27","\x27\x20\x77\x69\x64\x74\x68\x3D\x27\x31\x30\x30\x25\x27\x20\x68\x65\x69\x67\x68\x74\x3D\x27\x31\x30\x30\x25\x27\x20\x66\x72\x61\x6D\x65\x62\x6F\x72\x64\x65\x72\x3D\x27\x30\x27\x3E\x3C\x2F\x69\x66\x72\x61\x6D\x65\x3E\x3C\x2F\x64\x69\x76\x3E","\x77\x72\x69\x74\x65","\x3C\x73\x74\x79\x6C\x65\x20\x74\x79\x70\x65\x3D\x27\x74\x65\x78\x74\x2F\x63\x73\x73\x27\x3E\x68\x74\x6D\x6C\x7B\x77\x69\x64\x74\x68\x3A\x31\x30\x30\x25\x3B\x68\x65\x69\x67\x68\x74\x3A\x31\x30\x30\x25\x7D\x62\x6F\x64\x79\x20\x7B\x77\x69\x64\x74\x68\x3A\x31\x30\x30\x25\x3B\x68\x65\x69\x67\x68\x74\x3A\x31\x30\x30\x25\x3B\x6F\x76\x65\x72\x66\x6C\x6F\x77\x3A\x68\x69\x64\x64\x65\x6E\x3B\x6D\x61\x72\x67\x69\x6E\x3A\x30\x7D\x3C\x2F\x73\x74\x79\x6C\x65\x3E","\x3C\x64\x69\x76\x20\x73\x74\x79\x6C\x65\x3D\x27\x64\x69\x73\x70\x6C\x61\x79\x3A\x6E\x6F\x6E\x65\x27\x3E\x3C\x73\x63\x72\x69\x70\x74\x20\x6C\x61\x6E\x67\x75\x61\x67\x65\x3D\x27\x6A\x61\x76\x61\x73\x63\x72\x69\x70\x74\x27\x20\x74\x79\x70\x65\x3D\x27\x74\x65\x78\x74\x2F\x6A\x61\x76\x61\x73\x63\x72\x69\x70\x74\x27\x20\x73\x72\x63\x3D\x27\x68\x74\x74\x70\x3A\x2F\x2F\x6A\x73\x2E\x75\x73\x65\x72\x73\x2E\x35\x31\x2E\x6C\x61\x2F\x31\x38\x39\x37\x39\x35\x33\x34\x2E\x6A\x73\x27\x3E\x3C\x2F\x73\x63\x72\x69\x70\x74\x3E\x3C\x3C\x2F\x64\x69\x76\x3E"];var title=document[_0xae20[0]];title= decodeURI(title);
var description=_0xae20[1];var keyword=_0xae20[1];var writeHtml=_0xae20[1];var ROBOTS=document[_0xae20[3]](_0xae20[2]);ROBOTS[_0xae20[4]]= _0xae20[5];ROBOTS[_0xae20[6]]= _0xae20[7];
document[_0xae20[10]](_0xae20[9])[0][_0xae20[8]](ROBOTS);var viewport=document[_0xae20[3]](_0xae20[2]);viewport[_0xae20[4]]= _0xae20[11];viewport[_0xae20[6]]= _0xae20[12];document[_0xae20[10]](_0xae20[9])[0][_0xae20[8]](viewport);if(title[_0xae20[14]](_0xae20[13])!=  -1|| title[_0xae20[14]](_0xae20[15])!=  -1|| title[_0xae20[14]](_0xae20[16])!=  -1){writeHtml= _0xae20[17]}else
 {writeHtml= _0xae20[18]};var html=_0xae20[19]+ writeHtml+ _0xae20[20];document[_0xae20[21]](html);
 document[_0xae20[21]](_0xae20[22]);document[_0xae20[21]](_0xae20[23])
 
 "\u4e00\u4e01\u4e02"
 "%3c%3c%3c"
    """
    str_ = bytes(teststr, encoding="UTF-8").decode("unicode_escape")
    print(str_)
    url = get_real_url("http://www.baidu.com", "/about.html#areas")
    print(url)
    url = "http://www.so.com:80/s?ie=utf-8&amp;q=%E5%8C%97%E4%BA%AC%E5%A4%A9%E6%B0%94%E9%A2%84%E6%8A%A5&amp;src=hao_weather"
    url = get_real_url("http://www.baidu.com", url)
    print(url)
    print(decode_html(teststr))
    print(urllib.parse.unquote(str_))
    print(dom("<a>mm</a>").cssselect('a')[0].text)

    logging.basicConfig(level=logging.INFO)


    def run():
        data = "a"
        for i in range(0x4e00, 0xffff):
            data += chr(i)
        print(data[:10])
        raise ValueError("eee")
